//----------------------------------------------------------------------------
//
// File:		disk-image.hpp
// Date:		05-May-2015
// Programmer:	Marc Rousseau
//
// Description: A class to manipulate disk images
//
// Copyright (c) 2001-2004 Marc Rousseau, All Rights Reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
// Revision History:
//
//----------------------------------------------------------------------------

#ifndef DISK_IMAGE_HPP_
#define DISK_IMAGE_HPP_

#include <memory>
#include <vector>

#include "disk-track.hpp"

struct sDiskGapPair
{
	int                     count;
	int                     byte;
};

struct sDiskGap
{
	sDiskGapPair            filler;
	sDiskGapPair            sync;
	sDiskGapPair            clock;
};

struct sSectorInfo
{
	int                     LogicalCylinder;
	int                     LogicalHead;
	int                     LogicalSector;
	int                     Size;
	int                     DataMark;
};

class cDiskImage
{
	int                     NumTracks;
	int                     NumHeads;
	std::unique_ptr<
	std::vector<cDiskTrack>[]> Track;

public:

	static auto FormatTrack( track::Format format, const std::vector<sSectorInfo> &sectorInfo ) -> sDataBuffer;
	static auto FormatTrack( const sDiskGap gapInfo[ 5 ], const std::vector<sSectorInfo> &sectorInfo ) -> sDataBuffer;

	auto GetNumHeads( ) const -> int;

	auto GetNumTracks( ) const -> int;

	auto AllocateTracks( int, int ) -> bool;

	auto FormatDisk( int, int, int, track::Format ) -> bool;

	auto WriteTrack( int, int, track::Format, sDataBuffer ) -> bool;

	auto WriteSector( int, int, int, int, const void * ) -> bool;

	auto GetTrack( int, int ) -> iDiskTrack *;

	auto GetTrack( int, int ) const -> const iDiskTrack *;

	auto HasChanged( ) const -> bool;
	auto ClearChanged( ) -> void;
};

#endif
